// rustup install nightly && rustup default nightly
// rustc overflow.rs && ./overflow 127

// std::num::Saturating is an experimental API, so only available in nightly builds
#![feature(saturating_int_impl)]

use std::env;
use std::num::Wrapping;
use std::num::Saturating;

fn add_checked(n: i8) {
    // Normal rust integer operations: overflow-checked and panic
    let a: i8 = n;
    let b: i8 = 1;
    println!("{:?}", a + b);
}

fn add_wrapping(n: i8) {
    // Wrapping integer operations: two's complement behaviour
    let a: Wrapping<i8> = Wrapping(n);
    let b: Wrapping<i8> = Wrapping(1);
    println!("{:?}", a + b);
}

fn add_saturating(n: i8) {
    // Saturating integer operations: stop at the limits
    let a: Saturating<i8> = Saturating(n);
    let b: Saturating<i8> = Saturating(1);
    println!("{:?}", a + b);
}

fn main() {
    let args: Vec<String> = env::args().collect();
    let n = args[1].parse::<i8>().unwrap();
    add_wrapping(n);
    add_saturating(n);
    add_checked(n);
}

